/* GSI - general sound interface
 *
 * gsi_eventmasks.h
 *
 * ** GSI event specification **
 *
 *
 * 1996-1999 by W.H.Scholten
 *
 * This file is released to the public domain.
 */


#ifndef _GSI__eventmasks_h
#define _GSI__eventmasks_h

/*---------------------------------------------------------------------------*
 *      Event selection.
 *      (or just ignore unwanted events? how much is the overhead ?)
 *---------------------------------------------------------------------------*/
#define GSI_EVMASK_CHANNEL         1
#define GSI_EVMASK_SONG            2

#define GSI_EVMASK_MIDI_INFO       4 /* change of pan, volumes etc */
#define GSI_EVMASK_DEVICE_STATUS   8 /* were devices opened ok? */
#define GSI_EVMASK_CD             16

#define GSI_EVMASK_STREAM         32

#define GSI_EVMASK_ALL            0xffffffff

#endif /* _GSI__eventmasks_h */
