/* GSI - general sound interface
 *
 * mixer_data.h
 *
 * ** GSI data types specification **
 *
 *
 * 1996-1999 by W.H.Scholten
 *
 * This file is released to the public domain.
 */


#ifndef _GSI__mixer_data_h
#define _GSI__mixer_data_h

/*---------------------------------------------------------------------------*
 *      GSI data types etc.
 *---------------------------------------------------------------------------*/
/* Mixer channels (devices) */
/* 1. output: */
#define GSI_MASTER                    0
#define GSI_PCM                       1
#define GSI_SYNTH                     2
#define GSI_CD                        3

/* 2. input (&recording): */
#define GSI_LINE_IN                   4
#define GSI_MICROPHONE                5

#define GSIMIXER_NUM_DEVICE_TYPES     6
//#define GSI_MIXER_NUM_CHANNELS      6 /* 1 + last mixer channel */

#define GSI_MIXER_ALL                15 /* ?? to set volumes for all mixer channels for example */

#define _RECORD_DEVICE_OFFSET GSIMIXER_NUM_DEVICE_TYPES


#endif /* _GSI__mixer_data_h */
