/* Useful definitions
 * 
 * By W.H.Scholten 1998 - 2000
 *
 * This file is in the public domain.
 */


#ifndef _WHS__whs_defines_h
#define _WHS__whs_defines_h


/* Macros: */
#define min(x,y) ((x)>(y) ? (y): (x))
#define max(x,y) ((x)<(y) ? (y): (x))


#endif /* _WHS__whs_defines_h */
