/* GSI - general sound interface
 *
 * Event definitions (automatically generated from events.spec)
 */

#ifndef _GSI__gsi_events_h
#define _GSI__gsi_events_h

#define GSI_EV_SERVER_KILLED 0
#define GSI_EV_BAD_COMMAND 1
#define GSI_EV_NO_SUCH_SAMPLE_FILE 2
#define GSI_EV_NO_SUCH_SONG_FILE 3
#define GSI_EV_NO_SUCH_LIST_FILE 4
#define GSI_EV_SLOT_BUSY 5
#define GSI_EV_NO_INSTRUMENT_SPACE 6
#define GSI_EV_NOT_IMPLEMENTED 7
#define GSI_EV_OUT_OF_MEMORY 8
#define GSI_EV_OUT_OF_MEMORY_LOADING_SLOT 9
#define GSI_EV_BAD_SLOT 10
#define GSI_EV_PCM_BUSY 16
#define GSI_EV_SYNTH_BUSY 17
#define GSI_EV_CD_BUSY 18
#define GSI_EV_MIXER_BUSY 19
#define GSI_EV_CD_NO_DISK 20
#define GSI_EV_CD_NO_AUDIO_TRACKS 21
#define GSI_EV_CD_MOUNTED 22
#define GSI_EV_CD_NO_CHANGER 23
#define GSI_EV_CD_BAD_SLOT 24
#define GSI_EV_CD_EMPTY_SLOT 25
#define GSI_EV_NO_PCM 26
#define GSI_EV_NO_SYNTH 27
#define GSI_EV_NO_CD 28
#define GSI_EV_UNKNOWN_SAMPLE_FILEFORMAT 48
#define GSI_EV_UNKNOWN_SONG_FILEFORMAT 49
#define GSI_EV_NO_ROOM 56
#define GSI_EV_SHMATTACH_FAILED 63
#define GSI_EV_MIXER_GRABBED 64
#define GSI_EV_PCM_GRABBED 65
#define GSI_EV_SYNTH_GRABBED 66
#define GSI_EV_CD_GRABBED 67
#define GSI_EV_CD_SHARED 68
#define GSI_EV_MIXER_RELEASED 72
#define GSI_EV_PCM_RELEASED 73
#define GSI_EV_SYNTH_RELEASED 74
#define GSI_EV_CD_RELEASED 75
#define GSI_EV_SHARE_PERMISSIONS 95
#define GSI_EV_DEVICE_ADD 96
#define GSI_EV_CD_TRACK_LIST 128
#define GSI_EV_CD_STATUS 129
#define GSI_EV_MIXER_STATUS 130
#define GSI_EV_SLOT_INFO 131
#define GSI_EV_CLIENT_ID 132
#define GSI_EV_SHMID 133
#define GSI_EV_CD_OUTPUT 141
#define GSI_EV_SYNC 190
#define GSI_EV_SERVER_PROTOCOL_VERSION 191
#define GSI_EV_CHANNEL_STARTED 192
#define GSI_EV_CHANNEL_END 193
#define GSI_EV_CHANNEL_ENDALL 194
#define GSI_EV_SONG_STARTED 195
#define GSI_EV_SONG_END 196
#define GSI_EV_CD_TRACK_STARTED 197
#define GSI_EV_CD_TRACK_END 198
#define GSI_EV_CHANNEL_NO_SONG 200
#define GSI_EV_CHANNEL_NO_SAMPLE 201
#define GSI_EV_STREAM_UPDATE 207
#define GSI_EV_STREAM_UNDERFLOW 208
#define GSI_EV_SEND_INSTRUMENT 210
#define GSI_EV_SLOT_LOADED 224
#define GSI_EV_SONG_LOADED 225
#define GSI_EV_CD_INSERTED 240
#define GSI_EV_CD_EJECTED 241
#define GSI_EV_CD_CHANGED 242

#endif /* _GSI__gsi_events_h */
