/* GSI - general sound interface
 *
 * Command definitions (automatically generated from commands.spec)
 */

#ifndef _GSI__gsi_commands_h
#define _GSI__gsi_commands_h

#define GSI_CMD_PLAY 0
#define GSI_CMD_OVERWRITE 1
#define GSI_CMD_REPEAT 2
#define GSI_CMD_OVERWRITE_REPEAT 3
#define GSI_CMD_KILL_ALL 16
#define GSI_CMD_PLAY_SONG 17
#define GSI_CMD_STOP_SONG 18
#define GSI_CMD_CONTINUE_SONG 19
#define GSI_CMD_SONG_TO_POSITION 20
#define GSI_CMD_MIDI_CONTROL 21
#define GSI_CMD_CD_CONTROL 22
#define GSI_CMD_SET_MIXER_VOLUME 23
#define GSI_CMD_GET_MIXER_VOLUME 24
#define GSI_CMD_SET_MASTER_VOLUME 25
#define GSI_CMD_GET_MASTER_VOLUME 26
#define GSI_CMD_SET_MIXER_BALANCE 27
#define GSI_CMD_SET_CD_VOLUME 28
#define GSI_CMD_GET_CD_VOLUME 29
#define GSI_CMD_RECORD_SLOT 31
#define GSI_CMD_SET_OBSERVER_X 45
#define GSI_CMD_SET_OBSERVER_Z 46
#define GSI_CMD_SET_OBSERVER_Y 47
#define GSI_CMD_SET_OBSERVER_ANGLE 48
#define GSI_CMD_SET_OBSERVER_THETA 49
#define GSI_CMD_SET_OBSERVER_R 50
#define GSI_CMD_ADJUST_OBSERVER_X 51
#define GSI_CMD_ADJUST_OBSERVER_Z 52
#define GSI_CMD_ADJUST_OBSERVER_Y 53
#define GSI_CMD_ADJUST_OBSERVER_ANGLE 54
#define GSI_CMD_ADJUST_OBSERVER_THETA 55
#define GSI_CMD_ADJUST_OBSERVER_R 56
#define GSI_CMD_SET_OBSERVER_DX 57
#define GSI_CMD_SET_OBSERVER_DZ 58
#define GSI_CMD_SET_OBSERVER_DY 59
#define GSI_CMD_OBSERVER_ECHO_ON 60
#define GSI_CMD_OBSERVER_ECHO_OFF 61
#define GSI_CMD_OBSERVER_DOPPLER_ON 62
#define GSI_CMD_OBSERVER_DOPPLER_OFF 63
#define GSI_CMD_KILL 64
#define GSI_CMD_PAUSE 65
#define GSI_CMD_CONTINUE 66
#define GSI_CMD_LOOP 67
#define GSI_CMD_SET_VOLUME 68
#define GSI_CMD_SET_SAMPLE_POSITION 69
#define GSI_CMD_CHANGE_SAMPLE_POSITION 70
#define GSI_CMD_SET_SAMPLE_POSITION_SECONDS 71
#define GSI_CMD_CHANGE_SAMPLE_POSITION_SECONDS 72
#define GSI_CMD_SET_SAMPLE_FACTOR 73
#define GSI_CMD_SET_PAN 80
#define GSI_CMD_SET_DEPTH 81
#define GSI_CMD_SET_HEIGHT 82
#define GSI_CMD_SET_POSITION 83
#define GSI_CMD_ECHO_ON 94
#define GSI_CMD_ECHO_OFF 95
#define GSI_CMD_SET_X 128
#define GSI_CMD_SET_Z 129
#define GSI_CMD_SET_Y 130
#define GSI_CMD_SET_ANGLE 131
#define GSI_CMD_SET_THETA 132
#define GSI_CMD_SET_R 133
#define GSI_CMD_SET_ANGLE_R 134
#define GSI_CMD_ADJUST_X 135
#define GSI_CMD_ADJUST_Z 136
#define GSI_CMD_ADJUST_Y 137
#define GSI_CMD_ADJUST_ANGLE 138
#define GSI_CMD_ADJUST_THETA 139
#define GSI_CMD_ADJUST_R 140
#define GSI_CMD_ADJUST_ANGLE_R 141
#define GSI_CMD_SET_DX 142
#define GSI_CMD_SET_DZ 143
#define GSI_CMD_SET_DY 144
#define GSI_CMD_DOPPLER_ON 158
#define GSI_CMD_DOPPLER_OFF 159
#define GSI_CMD_SHMSLOT 192
#define GSI_CMD_SHMDETACH 193
#define GSI_CMD_SET_BASE_WIDTH 194
#define GSI_CMD_SET_SPEED_OF_SOUND 195
#define GSI_CMD_SEND_SONG_DATA 198
#define GSI_CMD_CONFIGURE_TRACK 199
#define GSI_CMD_SEND_SLOT_DATA 201
#define GSI_CMD_STREAM 202
#define GSI_CMD_CONFIGURE_SLOT 203
#define GSI_CMD_GET_SLOT_INFO 207
#define GSI_CMD_COPY_SLOT 208
#define GSI_CMD_LOAD_INSTRUMENT_DATA_FROM_FILE 209
#define GSI_CMD_LOAD_INSTRUMENT_FROM_FILE 210
#define GSI_CMD_LOAD_INSTRUMENT_DATA 211
#define GSI_CMD_LOAD_SONG_DATA_FROM_FILE 212
#define GSI_CMD_LOAD_SONG_FROM_FILE 213
#define GSI_CMD_LOAD_SONG_DATA 214
#define GSI_CMD_LOAD_SOUND_DATA_FROM_FILE 215
#define GSI_CMD_LOAD_SOUND_DATA 216
#define GSI_CMD_LOAD_SOUND_FROM_FILE 217
#define GSI_CMD_FREE_SOUND 218
#define GSI_CMD_LOAD_SOUND 219
#define GSI_CMD_AUTO_FREE_SOUND 220
#define GSI_CMD_FREE_ALL 221
#define GSI_CMD_LOAD_ALL 222
#define GSI_CMD_SET_FILE_NAME 223
#define GSI_CMD_LOAD_SOUND_LIST 224
#define GSI_CMD_SET_WORKING_DIRECTORY 226
#define GSI_CMD_ADD_DEVICE 229
#define GSI_CMD_RELEASE_MIXER 231
#define GSI_CMD_GRAB_MIXER 232
#define GSI_CMD_AUTO_CD 233
#define GSI_CMD_RELEASE_CD 234
#define GSI_CMD_SHARE_CD 235
#define GSI_CMD_GRAB_CD 236
#define GSI_CMD_AUTO_SYNTH 237
#define GSI_CMD_RELEASE_SYNTH 238
#define GSI_CMD_GRAB_SYNTH 239
#define GSI_CMD_AUTO_PCM 240
#define GSI_CMD_RELEASE_PCM 241
#define GSI_CMD_GRAB_PCM 242
#define GSI_CMD_USE_BEST_PCM_SETTINGS 243
#define GSI_CMD_SET_PCM_FRAGSIZE 244
#define GSI_CMD_SET_PCM_TYPE 245
#define GSI_CMD_SET_PCM_CHANNELS 246
#define GSI_CMD_SET_PCM_SAMPLERATE 247
#define GSI_CMD_GET_CLIENT_ID 248
#define GSI_CMD_DISABLE_EVENTS 249
#define GSI_CMD_ENABLE_EVENTS 250
#define GSI_CMD_CONTINUE_ALL 252
#define GSI_CMD_PAUSE_ALL 253
#define GSI_CMD_SYNC 254
#define GSI_CMD_DEBUG 255

#endif /* _GSI__gsi_commands_h */
