/* GSI - general sound interface
 *
 * device.h - device abstraction
 *
 * 1999 by W.H.Scholten
 *
 * This file is released to the public domain.
 */


#ifndef _GSI__device_h
#define _GSI__device_h

#include <sys/time.h>


/*---------------------------------------------------------------------------*
 *      generic device (common to cd, pcm, synth)
 *---------------------------------------------------------------------------*/
typedef struct {
	char *device; /* the device to open, e.g. /dev/cdrom */
	char id[32];
} gsi_generic_device_t;

typedef struct {
	int  dev; /* weg ? */
	int  fd;
	int  fd_ct;
} gsi_generic_devicedata_t;


#endif /* _GSI__device_h */
