/* WHS misc
 *
 * p_signal.h
 *
 * 1999 by W.H.Scholten
 *
 * This file is released to the public domain.
 */


#ifndef _WHS__p_signal_h
#define _WHS__p_signal_h

#include <signal.h>
typedef void    sigfunc_t(int);

extern sigfunc_t *p_signal(int signo, sigfunc_t *func);
extern sigfunc_t *p_signal_restart(int signo, sigfunc_t *func, int restart);


#endif /* _WHS__p_signal_h */
