/* GSI - general sound interface
 *
 * synth_data.h
 *
 * ** GSI data types specification **
 *
 *
 * 1996-1999 by W.H.Scholten
 *
 * This file is released to the public domain.
 */


#ifndef _GSI__synth_data_h
#define _GSI__synth_data_h

/*---------------------------------------------------------------------------*
 *      GSI data types etc.
 *---------------------------------------------------------------------------*/
/* Time types: */
#define GSI_SONG_MILLISECONDS         0
#define GSI_SONG_TICKS                1

#define GSI_SAMPLE_MICROSECONDS       0
#define GSI_SAMPLE_METASAMPLES        1 /* a meta sample is 4 bytes for stereo 16 bit etc */

/* Synthesizer instrument (music sample) types: */
#define GSI_INSTRUMENT_OPL2                1
#define GSI_INSTRUMENT_OPL3                2
#define GSI_INSTRUMENT_GUS                 3
#define GSI_INSTRUMENT_INTERWAVE           4
#define GSI_INSTRUMENT_AWE                 5


#endif /* _GSI__synth_data_h */
