/* GSI configuration created by Configure.sh */
#define SYSCONFIG_LITTLE_ENDIAN 1234
#define SYSCONFIG_BIG_ENDIAN 4321
#define  SYSCONFIG_LITTLE_ENDIAN_MACHINE
#define  SYSCONFIG_MACHINE_BYTE_ORDER  SYSCONFIG_LITTLE_ENDIAN
#define  USE_OSS 1
#define  USE_NAS 0
#define  USE_ALSA 0
#define  USE_FBSD 0
#define  USE_DEV_AUDIO 0
#define  USE_ULTRASOUND 0
#define  USE_OPENBSD_AUDIO 0
#define  OSS_OPL_INSTRUMENT_DIR "/etc/midi"
#define  USE_GSI_DRUMS 1
#define  GUS_INSTRUMENT_DIR "/dos/ultrasnd/midi"
#define  GUS_HAVE_ALL_INSTRUMENTS
#define  USE_MIDI 1
#define  USE_OPL3_SYNTH 1
#define  USE_GUS_SYNTH 1
#define  USE_AWE_SYNTH 0
#define  USE_SYSV_SHM 1
