/* Configuration definitions
 * 
 * By W.H.Scholten 1996 - 1999
 *
 * This file is public domain.
 */


#ifndef _GSI__gsi_config_h
#define _GSI__gsi_config_h

#include <gsi/config.h>
#define USE_SOCKETS

/* The /etc/services protocol name, needed both by GSI server and interface: */
#define GSI_SERVICE   "gsi_server"


/* The AF_LOCAL socket name; Note that it must be 13 characters or less: */
//#define GSI_LOCAL_SOCKET_NAME  "/tmp/gsi"
// Temporary (so we can use devel & stable at the same time):
#define GSI_LOCAL_SOCKET_NAME  "/tmp/GSI"


#endif /* _GSI__gsi_config_h */
