/* GSI - general sound interface
 *
 * Event names (automatically generated from commands.spec)
 */

static char *gsi_event_name[256] = {
	"GSI_EV_SERVER_KILLED",
	"GSI_EV_BAD_COMMAND",
	"GSI_EV_NO_SUCH_SAMPLE_FILE",
	"GSI_EV_NO_SUCH_SONG_FILE",
	"GSI_EV_NO_SUCH_LIST_FILE",
	"GSI_EV_SLOT_BUSY",
	"GSI_EV_NO_INSTRUMENT_SPACE",
	"GSI_EV_NOT_IMPLEMENTED",
	"GSI_EV_OUT_OF_MEMORY",
	"GSI_EV_OUT_OF_MEMORY_LOADING_SLOT",
	"GSI_EV_BAD_SLOT",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"GSI_EV_PCM_BUSY",
	"GSI_EV_SYNTH_BUSY",
	"GSI_EV_CD_BUSY",
	"GSI_EV_MIXER_BUSY",
	"GSI_EV_CD_NO_DISK",
	"GSI_EV_CD_NO_AUDIO_TRACKS",
	"GSI_EV_CD_MOUNTED",
	"GSI_EV_CD_NO_CHANGER",
	"GSI_EV_CD_BAD_SLOT",
	"GSI_EV_CD_EMPTY_SLOT",
	"GSI_EV_NO_PCM",
	"GSI_EV_NO_SYNTH",
	"GSI_EV_NO_CD",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"GSI_EV_UNKNOWN_SAMPLE_FILEFORMAT",
	"GSI_EV_UNKNOWN_SONG_FILEFORMAT",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"GSI_EV_NO_ROOM",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"GSI_EV_SHMATTACH_FAILED",
	"GSI_EV_MIXER_GRABBED",
	"GSI_EV_PCM_GRABBED",
	"GSI_EV_SYNTH_GRABBED",
	"GSI_EV_CD_GRABBED",
	"GSI_EV_CD_SHARED",
	"invalid",
	"invalid",
	"invalid",
	"GSI_EV_MIXER_RELEASED",
	"GSI_EV_PCM_RELEASED",
	"GSI_EV_SYNTH_RELEASED",
	"GSI_EV_CD_RELEASED",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"GSI_EV_SHARE_PERMISSIONS",
	"GSI_EV_DEVICE_ADD",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"GSI_EV_CD_TRACK_LIST",
	"GSI_EV_CD_STATUS",
	"GSI_EV_MIXER_STATUS",
	"GSI_EV_SLOT_INFO",
	"GSI_EV_CLIENT_ID",
	"GSI_EV_SHMID",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"GSI_EV_CD_OUTPUT",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"GSI_EV_SYNC",
	"GSI_EV_SERVER_PROTOCOL_VERSION",
	"GSI_EV_CHANNEL_STARTED",
	"GSI_EV_CHANNEL_END",
	"GSI_EV_CHANNEL_ENDALL",
	"GSI_EV_SONG_STARTED",
	"GSI_EV_SONG_END",
	"GSI_EV_CD_TRACK_STARTED",
	"GSI_EV_CD_TRACK_END",
	"invalid",
	"GSI_EV_CHANNEL_NO_SONG",
	"GSI_EV_CHANNEL_NO_SAMPLE",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"GSI_EV_STREAM_UPDATE",
	"GSI_EV_STREAM_UNDERFLOW",
	"invalid",
	"GSI_EV_SEND_INSTRUMENT",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"GSI_EV_SLOT_LOADED",
	"GSI_EV_SONG_LOADED",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"GSI_EV_CD_INSERTED",
	"GSI_EV_CD_EJECTED",
	"GSI_EV_CD_CHANGED",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
};

/* EOF event_names.h */
