/* GSI - general sound interface
 *
 * version.h
 *
 * 1996 - 2000 by W.H.Scholten
 *
 * This file is in the public domain.
 */


#ifndef _GSI__version_h
#define _GSI__version_h

#define GSI_VERSION_STRING      "0.9.4"
#define GSI_SERVER_STRING       "gsi_server "GSI_VERSION_STRING

#define GSI_PROTOCOL_VERSION_BCD             0x000902
/*      = version of the GSI version as a whole in which the protocol was
 *        introduced. */

#define GSI_PROTOCOL_COMPATIBLE_VERSION_BCD  0x000902
/*      = lowest protocol version still compatible */



#define GSI_NAS_INTERFACE_VERSION      "0.1.9"
#define GSI_NAS_INTERFACE_BCD_VERSION  0x000109
#define GSI_NAS_INTERFACE_STRING       "gsi NAS interface "GSI_NAS_INTERFACE_VERSION


#endif /* _GSI__version_h */
