/* GSI - general sound interface
 *
 * Command names (automatically generated from commands.spec)
 */

static char *gsi_command_name[256] = {
	"GSI_CMD_PLAY",
	"GSI_CMD_OVERWRITE",
	"GSI_CMD_REPEAT",
	"GSI_CMD_OVERWRITE_REPEAT",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"GSI_CMD_KILL_ALL",
	"GSI_CMD_PLAY_SONG",
	"GSI_CMD_STOP_SONG",
	"GSI_CMD_CONTINUE_SONG",
	"GSI_CMD_SONG_TO_POSITION",
	"GSI_CMD_MIDI_CONTROL",
	"GSI_CMD_CD_CONTROL",
	"GSI_CMD_SET_MIXER_VOLUME",
	"GSI_CMD_GET_MIXER_VOLUME",
	"GSI_CMD_SET_MASTER_VOLUME",
	"GSI_CMD_GET_MASTER_VOLUME",
	"GSI_CMD_SET_MIXER_BALANCE",
	"GSI_CMD_SET_CD_VOLUME",
	"GSI_CMD_GET_CD_VOLUME",
	"invalid",
	"GSI_CMD_RECORD_SLOT",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"GSI_CMD_SET_OBSERVER_X",
	"GSI_CMD_SET_OBSERVER_Z",
	"GSI_CMD_SET_OBSERVER_Y",
	"GSI_CMD_SET_OBSERVER_ANGLE",
	"GSI_CMD_SET_OBSERVER_THETA",
	"GSI_CMD_SET_OBSERVER_R",
	"GSI_CMD_ADJUST_OBSERVER_X",
	"GSI_CMD_ADJUST_OBSERVER_Z",
	"GSI_CMD_ADJUST_OBSERVER_Y",
	"GSI_CMD_ADJUST_OBSERVER_ANGLE",
	"GSI_CMD_ADJUST_OBSERVER_THETA",
	"GSI_CMD_ADJUST_OBSERVER_R",
	"GSI_CMD_SET_OBSERVER_DX",
	"GSI_CMD_SET_OBSERVER_DZ",
	"GSI_CMD_SET_OBSERVER_DY",
	"GSI_CMD_OBSERVER_ECHO_ON",
	"GSI_CMD_OBSERVER_ECHO_OFF",
	"GSI_CMD_OBSERVER_DOPPLER_ON",
	"GSI_CMD_OBSERVER_DOPPLER_OFF",
	"GSI_CMD_KILL",
	"GSI_CMD_PAUSE",
	"GSI_CMD_CONTINUE",
	"GSI_CMD_LOOP",
	"GSI_CMD_SET_VOLUME",
	"GSI_CMD_SET_SAMPLE_POSITION",
	"GSI_CMD_CHANGE_SAMPLE_POSITION",
	"GSI_CMD_SET_SAMPLE_POSITION_SECONDS",
	"GSI_CMD_CHANGE_SAMPLE_POSITION_SECONDS",
	"GSI_CMD_SET_SAMPLE_FACTOR",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"GSI_CMD_SET_PAN",
	"GSI_CMD_SET_DEPTH",
	"GSI_CMD_SET_HEIGHT",
	"GSI_CMD_SET_POSITION",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"GSI_CMD_ECHO_ON",
	"GSI_CMD_ECHO_OFF",
	"(CURRENT) GSI_CMD_KILL",
	"(CURRENT) GSI_CMD_PAUSE",
	"(CURRENT) GSI_CMD_CONTINUE",
	"(CURRENT) GSI_CMD_LOOP",
	"(CURRENT) GSI_CMD_SET_VOLUME",
	"(CURRENT) GSI_CMD_SET_SAMPLE_POSITION",
	"(CURRENT) GSI_CMD_CHANGE_SAMPLE_POSITION",
	"(CURRENT) GSI_CMD_SET_SAMPLE_POSITION_SECONDS",
	"(CURRENT) GSI_CMD_CHANGE_SAMPLE_POSITION_SECONDS",
	"(CURRENT) GSI_CMD_SET_SAMPLE_FACTOR",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"(CURRENT) GSI_CMD_SET_PAN",
	"(CURRENT) GSI_CMD_SET_DEPTH",
	"(CURRENT) GSI_CMD_SET_HEIGHT",
	"(CURRENT) GSI_CMD_SET_POSITION",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"(CURRENT) GSI_CMD_ECHO_ON",
	"(CURRENT) GSI_CMD_ECHO_OFF",
	"GSI_CMD_SET_X",
	"GSI_CMD_SET_Z",
	"GSI_CMD_SET_Y",
	"GSI_CMD_SET_ANGLE",
	"GSI_CMD_SET_THETA",
	"GSI_CMD_SET_R",
	"GSI_CMD_SET_ANGLE_R",
	"GSI_CMD_ADJUST_X",
	"GSI_CMD_ADJUST_Z",
	"GSI_CMD_ADJUST_Y",
	"GSI_CMD_ADJUST_ANGLE",
	"GSI_CMD_ADJUST_THETA",
	"GSI_CMD_ADJUST_R",
	"GSI_CMD_ADJUST_ANGLE_R",
	"GSI_CMD_SET_DX",
	"GSI_CMD_SET_DZ",
	"GSI_CMD_SET_DY",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"GSI_CMD_DOPPLER_ON",
	"GSI_CMD_DOPPLER_OFF",
	"(CURRENT) GSI_CMD_SET_X",
	"(CURRENT) GSI_CMD_SET_Z",
	"(CURRENT) GSI_CMD_SET_Y",
	"(CURRENT) GSI_CMD_SET_ANGLE",
	"(CURRENT) GSI_CMD_SET_THETA",
	"(CURRENT) GSI_CMD_SET_R",
	"(CURRENT) GSI_CMD_SET_ANGLE_R",
	"(CURRENT) GSI_CMD_ADJUST_X",
	"(CURRENT) GSI_CMD_ADJUST_Z",
	"(CURRENT) GSI_CMD_ADJUST_Y",
	"(CURRENT) GSI_CMD_ADJUST_ANGLE",
	"(CURRENT) GSI_CMD_ADJUST_THETA",
	"(CURRENT) GSI_CMD_ADJUST_R",
	"(CURRENT) GSI_CMD_ADJUST_ANGLE_R",
	"(CURRENT) GSI_CMD_SET_DX",
	"(CURRENT) GSI_CMD_SET_DZ",
	"(CURRENT) GSI_CMD_SET_DY",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"invalid",
	"GSI_CMD_SHMSLOT",
	"GSI_CMD_SHMDETACH",
	"GSI_CMD_SET_BASE_WIDTH",
	"GSI_CMD_SET_SPEED_OF_SOUND",
	"invalid",
	"invalid",
	"GSI_CMD_SEND_SONG_DATA",
	"GSI_CMD_CONFIGURE_TRACK",
	"invalid",
	"GSI_CMD_SEND_SLOT_DATA",
	"GSI_CMD_STREAM",
	"GSI_CMD_CONFIGURE_SLOT",
	"invalid",
	"invalid",
	"invalid",
	"GSI_CMD_GET_SLOT_INFO",
	"GSI_CMD_COPY_SLOT",
	"GSI_CMD_LOAD_INSTRUMENT_DATA_FROM_FILE",
	"GSI_CMD_LOAD_INSTRUMENT_FROM_FILE",
	"GSI_CMD_LOAD_INSTRUMENT_DATA",
	"GSI_CMD_LOAD_SONG_DATA_FROM_FILE",
	"GSI_CMD_LOAD_SONG_FROM_FILE",
	"GSI_CMD_LOAD_SONG_DATA",
	"GSI_CMD_LOAD_SOUND_DATA_FROM_FILE",
	"GSI_CMD_LOAD_SOUND_DATA",
	"GSI_CMD_LOAD_SOUND_FROM_FILE",
	"GSI_CMD_FREE_SOUND",
	"GSI_CMD_LOAD_SOUND",
	"GSI_CMD_AUTO_FREE_SOUND",
	"GSI_CMD_FREE_ALL",
	"GSI_CMD_LOAD_ALL",
	"GSI_CMD_SET_FILE_NAME",
	"GSI_CMD_LOAD_SOUND_LIST",
	"invalid",
	"GSI_CMD_SET_WORKING_DIRECTORY",
	"invalid",
	"invalid",
	"GSI_CMD_ADD_DEVICE",
	"invalid",
	"GSI_CMD_RELEASE_MIXER",
	"GSI_CMD_GRAB_MIXER",
	"GSI_CMD_AUTO_CD",
	"GSI_CMD_RELEASE_CD",
	"GSI_CMD_SHARE_CD",
	"GSI_CMD_GRAB_CD",
	"GSI_CMD_AUTO_SYNTH",
	"GSI_CMD_RELEASE_SYNTH",
	"GSI_CMD_GRAB_SYNTH",
	"GSI_CMD_AUTO_PCM",
	"GSI_CMD_RELEASE_PCM",
	"GSI_CMD_GRAB_PCM",
	"GSI_CMD_USE_BEST_PCM_SETTINGS",
	"GSI_CMD_SET_PCM_FRAGSIZE",
	"GSI_CMD_SET_PCM_TYPE",
	"GSI_CMD_SET_PCM_CHANNELS",
	"GSI_CMD_SET_PCM_SAMPLERATE",
	"GSI_CMD_GET_CLIENT_ID",
	"GSI_CMD_DISABLE_EVENTS",
	"GSI_CMD_ENABLE_EVENTS",
	"invalid",
	"GSI_CMD_CONTINUE_ALL",
	"GSI_CMD_PAUSE_ALL",
	"GSI_CMD_SYNC",
	"GSI_CMD_DEBUG",
};

/* EOF command_names.h */
