/* Debug definitions
 * 
 * By W.H.Scholten 1996,1997,1998
 *
 * This file is public domain.
 */


#ifndef _GSI__gsi_debug_h
#define _GSI__gsi_debug_h

//#define DEBUG_NAME "DEBUG_LEVEL"
#include <gsi/whs_debug.h>


#endif /* _GSI__gsi_debug_h */
